/* 

============================================================
+--------------------+
|attribute mask table|
+--------------------+

                                 tile_x & 0x02
                     # 0x00             | 0x10             |
               ======+==================+==================+
                0x00 # 0x03             | 0x0c (0x03 << 2) |
tile_y & 0x02  ------+------------------+------------------+
                0x10 # 0x30 (0x03 << 4) | 0xc0 (0x03 << 6) |
               ======+==================+==================+

attribute mask can calculate by 
  uint8 shift = ((tile_y & 0x02) << 1) | (tile_x & 0x02);
  uint8 mask = 0x03 << shift;

and palette index can calculate by
  (((VRAM(attrib_addr) & mask) << 2) >> shift);
  
  *NOTE* palette index in 0, 4, 8, 12. 
*/

/* ============================================================ */
void
#if defined (RENDER_BG_MMC5)
#  ifdef RENDER_BG_SKIP
NES_PPU::render_bg_mmc5_skip ()
#  else
NES_PPU::render_bg_mmc5 (uint16 *buf)
#  endif
#elif defined (RENDER_BG_MMC2)
#  ifdef RENDER_BG_SKIP
NES_PPU::render_bg_mmc2_skip ()
#  else
NES_PPU::render_bg_mmc2 (uint16 *buf)
#  endif
#elif defined (RENDER_BG_BANKSWITCH_PER_TILE)
NES_PPU::render_bg_bankswitch_per_tile (uint16 *buf, float CYCLES_PER_DRAW)
#else
NES_PPU::render_bg_regular (uint16 *buf)
#endif
{
  uint16 *p;
  int loop_before_switch;
  int loop_after_switch;
  
  uint8 *s;
  
  uint32 tile_x; // pixel coords within nametable
  uint32 tile_y;
  uint32 tile_y_bit;
  uint32 name_addr;
  
  uint32 pattern_addr;
  uint16 *pattern_palette;
  
  uint32 attrib_addr_base;
  
  tile_x = (loopy_v & 0x001F);
  tile_y = (loopy_v & 0x03E0) >> 5;
  tile_y_bit = (loopy_v & 0x7000) >> 12;
  
  name_addr = 0x2000 + (loopy_v & 0x0FFF);
  attrib_addr_base = 0x2000 + (loopy_v & 0x0C00) + 0x03C0 + ((tile_y & 0xFFFC)<<1);
  
#ifndef RENDER_BG_SKIP  
  p = buf       + (SIDE_MARGIN - loopy_x);
  s = solid_buf + (SIDE_MARGIN - loopy_x); 
  
  /* adjust before loop */
  p -= 8;
  s -= 8;
#endif
  
#ifdef RENDER_BG_MMC5
  parent_NES->mapper->PPU_Latch_RenderScreen(1, 0);
#endif
  
  loop_before_switch = 0x1f - tile_x;
  loop_after_switch = 32 - loop_before_switch;
  
#include "render_tile.cpp"
  while (loop_before_switch--)
  {
    ++tile_x;
    ++name_addr;
#include "render_tile.cpp"
  }
  
  tile_x -= 0x1f;
  name_addr ^= 0x41f;
  attrib_addr_base ^= 0x400;
  
  while (loop_after_switch--)
  {
#include "render_tile.cpp"
    ++tile_x;
    ++name_addr;
  }
  
#ifndef RENDER_BG_SKIP  
  if(bg_clip_left8())
  {
    /* clip left 8 pixels */
    p = buf + SIDE_MARGIN;
    *p++ = 0; *p++ = 0;
    *p++ = 0; *p++ = 0;
    *p++ = 0; *p++ = 0;
    *p++ = 0; *p++ = 0;
    
    s = solid_buf + SIDE_MARGIN;
    *s++ = 0; *s++ = 0;
    *s++ = 0; *s++ = 0;
    *s++ = 0; *s++ = 0;
    *s++ = 0; *s++ = 0;
  }
#endif
}


